{% if params %}
<div class="panel-footer">
    <div class="form-inline">
        {% for k, v in params.items %}
            <div class="form-group">
                <label for="{{ k }}_param" class="control-label col-sm-4">{{ v.label }}:</label>
                <div class="col-sm-7">
                    <input type="text" data-param="{{ k }}" class="param form-control" name="{{ k }}_param" id="{{ k }}_param" placeholder="parameter" value="{{ v.val }}" />
                </div>
                <div class="col-sm-1"></div>
            </div>
        {% endfor %}
    </div>
</div>
{% endif %}
