# Generated by Django 3.0.8 on 2020-12-18 06:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('explorer', '0009_auto_20201009_0547'),
    ]

    operations = [
        migrations.AlterField(
            model_name='query',
            name='sql',
            field=models.TextField(),
        ),
    ]
